%% Build PLS model from the command line

clear;clc
load 'basic data'
whos

% build 5 LV PLS model
myModel = pls(spec1Data, concVals, 5);


%% Build PLS model from the command line with options

% create options structure for PLS function
myOptions = pls('options');
% turn off command line display output
myOptions.display = 'off';
% turn off plots
myOptions.plots = 'none';
% set preprocessing for x-block and y-block
myOptions.preprocessing = {'mean center' 'mean center'};

myModel = pls(spec1Data, concVals, 5, myOptions);

%% Perform cross validation "after the fact"

myOptionsCV = crossval('options');
myOptionsCV.display = 'off';

% add cross validation data (out to 20 LVs) to the existing model myModel
myModelCV = crossval(spec1Data, concVals, myModel,  {'vet' 5}, 20, myOptionsCV);

%% Cross validation "before the fact"

myResultsCV = crossval(spec1Data, concVals, 'pls', {'vet' 5}, 20, myOptionsCV);

%% A documented goodie

% throw in a test set to get RMSEP metrics
myOptionsCV.testx = spec2Data;
myOptionsCV.testy = concVals;
myModelCV = crossval(spec1Data, concVals, myModel, {'vet' 5}, 20, myOptionsCV);

%% Previous results - data type

class(myModel)
class(myModelCV)
class(myResultsCV)


%% DataSet object - arch demo data

% take a look at the .classlookup field for the arch dataset
load arch
arch.classlookup{1,1}
classsummary(arch)

%% extract samples from class "BL"

% use dynamic referencing to create a dataset object of samples only from
% the BL quarry

archBL = arch.('BL');
classsummary(archBL)

%% create subsets consisting of a single class, except for class 0 and "K"

myClasses = arch.classlookup{1,1}(3:end,2)
myClasses = myClasses(:)'
% make myClasses a row vector so that we can loop over it

%%

for className = myClasses
  archSub.(char(className)) = arch.(char(className));
end

archSub

% so . . . why not class K?

arch.('K')

%% Model objects

clear;clc;load nir_data
myModel = evrimodel('pls')

% note the properties

% here are the valid inputs for the model object in its current state
myModel.inputs

%% populate x-block and y-block, set preprocessing, 

myModel.x = spec1;
myModel.y = conc;
myModel.y.include{2} = 3;
myModel.ncomp = 5;
myModel.options.preprocessing = {'mean center' 'mean center'};

myModel
myModel.options

%% calibrate the model and generate some plots

myModel = myModel.calibrate;

% here are the valid inputs for the model object in its current
% (calibrated) state

myModel.inputs

% Here's how we get those cool plots!
% plot loadings
myModel.plotloads

% note the x-axis values - carried over from the dataset where there is 
% an axis scale for model 2 (variables)

% plot scores
myModel.plotscores



%% cross validate - start from scratch, #LVs = 5, cross validate out to 20

clc;close all
myModel = evrimodel('pls');
myModel.x = spec1;
myModel.y = conc;
myModel.y.include{2} = 3;
myModel.ncomp = 5;
myModel.options.preprocessing = {'mean center' 'mean center'};

myModel = myModel.crossvalidate({'vet' 5}, 20);

% cvi = {'vet' 5}

% plot eigenvalues
myModel.ploteigen
myModel.ncomp

% while stats are carried out to 20 LVs, this is a model with 5 LVs


%% Scripting the Analysis window - launch

clear;load arch

myObj = evrigui('analysis');
myObj.interface
% list of available properties/methods for the evrigui Analysis object


%% Scripting the Analysis window - configure PCA model

% use the first 63 samples of arch to build a calibration model, and apply
% it to the remaining 12 samples

myObj.setXblock(arch(1:63,:));
myObj.setXblockVal(arch(64:end,:));
myObj.setXPreprocessing('autoscale');
myObj.setCrossvalidation({'vet' 6});
myObj.setMethod('pca');

%% Scripting the Analysis window - build PCA model and review

myObj.calibrate;

myObj.getButtons'
% these are the available "buttons" that you can emulate pressing using the
% .pressButton method


% here we create an eigenvalues plot

myObj.pressButton('ploteigen')


%% Scripting the Analysis window - refine the number of components and rebuild

myObj.setComponents(4);
myObj.calibrate;
myObj.pressButton('plotscores')

%% Scripting the Analysis window - modify the scores plot

% Show calibration with validation
%
% Changing this setting is actually tied in with the Analysis window
% First we need the figure handle of the Analysis window

analysisFig = findobj(allchild(0), 'tag', 'analysis');

% The following function toggles between the two different states

analysis('showcalwithtest', [], [], guidata(analysisFig));

%% Scripting the Analysis window - save the model and prediction object to the workspace

% extract the model and prediction objects from the interface and save them
% to variables in the workspace

myModel = myObj.getModel;
myPrediction = myObj.getPrediction;



%% Report Writer example - preparation

% Let's build a model and generate lots of plots!

clear;clc;close all
tic
% load data, set up PLSDA model, calibrate
load arch

myModel = evrimodel('plsda');
myModel.x = arch(1:63,:);
myModel.ncomp = 4;
myModel.options.preprocessing = {'autoscale' 'autoscale'};

myModel = myModel.calibrate;

% end of calibration

% the next two lines are so that we can include reduced Q and T^2 in our
% scores dataset

ops = plotscores('options');
ops.reducedstats = {'q' 't2'};

% model.plotscores without an assigned output produces a scores plot; 
% assigning an output returns a dataset object

myScores = myModel.plotscores(ops);
colNames = str2cell(myScores.label{2,1});
colNames = colNames(:)

colArray = [num2cell([1:length(colNames)]') colNames]


%% Report Writer example - generate the plots part I

%  I/O: filename = reportwriter(target, modelcell, comments, options); 

% bunches of scores plots
newScoresFig = plotgui('new', myScores, 'columns', 'viewclasses', 1);
legend('show', 'Location', 'best')

% this will be our "template" figure - now we will generate individual
% plots of the columns we wish to display

colInds = [1:4 7 8 12:17 20:24];

% now for each new plot - built upon the first - select which column to
% plot

getappdata(newScoresFig, 'axismenuindex')

%% Report Writer example - generate the plots part II

figCell = cell(1, length(colInds)+4);
% loads plot:                 +1
% confusion heat map          +1
% infobox with confusion data +1
% holder for model            +1

% in this section we loop over each of the columns we wish to plot
% for each iteration, make a duplicate of the template plot (scores on LV1)
% and modify it accordingly with the axismenuvalues appdata property

figCell{1} = newScoresFig;
newScoresFig.Name = colNames{1};
counter = 2;
for curInd = colInds(2:end)
  curFig = plotgui('duplicate', newScoresFig);
  curFig.Name = colNames{curInd};
  figCell{counter} = curFig;
  counter = counter + 1;
  plotgui('update', 'figure', curFig, 'axismenuvalues', {0 curInd 0});
  legend('show', 'Location', 'best')
end

% create loadings plot
myModel.plotloads;newLoadsFig = gcf;figCell{counter} = newLoadsFig;
counter = counter + 1;

%% ReportWriter example part III - modify loadings plot

plotgui('update', 'figure', newLoadsFig, 'axismenuvalues', {0 1:4 0}, ...
  'plottype', 'line', 'ViewLabels', 1);
legend('show', 'Location', 'best')
% overlay all of the loadings

%% Report Writer example  part IV - create confusion heat map

[confusiontab, classnames] = confusiontable(myModel);

newFigHdl = figure;
newHeatMapHdl = heatmap(classnames, classnames,confusiontab(1:(end-1),:));
figCell{counter} = newFigHdl;
counter = counter + 1;
newHeatMapHdl.FontSize = 16;
newFigHdl.Name = 'Confusion Heatmap';

xlabel('Actual Class')
ylabel('Predicted Class')
title('Confusion Table for arch')

%% Report Writer example  part V - create an infobox with confusion summaries

% extract most probable results
contentMostProbable = plsda_guifcn('getcontent', myModel, [], 'mostprobable');
% extract strict results
contentStrict = plsda_guifcn('getcontent', myModel, [], 'strict');

% the rest of this is formatting
maxLength = max(cellfun('length', contentMostProbable));
breakline = {repmat('-', 1, maxLength)};
% combine most probable and strict results
content = [contentMostProbable {' '} breakline contentStrict];
newIBHdl = infobox(content, struct(...
  'visible', 'on', ...
  'fontname', 'courier', ...
  'fontsize', 9, ...
  'figurename', 'Confusion Matrix'));
newIBHdl.IntegerHandle = 'on'; 
% we do this because the report writer requires figures with integer
% handle values

textBoxHdl = findobj(allchild(newIBHdl), 'tag', 'listbox');
textBoxHdl.FontWeight = 'bold';

figCell{counter} = newIBHdl;

%% Report Writer - final steps

figCell{end} = myModel;

reportwriter('html', figCell, 'TPS Report');

%% Using plotgui commands to make a movie
% Revisit the PLSDA model from the arch dataset

clear;clc;close all

% build PLSDA model from first 63 samples of arch

load arch

myModel = evrimodel('plsda');
myModel.x = arch(1:63,:);
myModel.ncomp = 4;
myModel.options.preprocessing = {'autoscale' 'autoscale'};

myModel = myModel.calibrate;

ops = plotscores('options');
ops.reducedstats = {'q' 't2'};

myScores = myModel.plotscores(ops);

% Recall that there's 44 columns in the scores dataset object.  Let's
% create a figure with 4 plots and we will loop through all of the columns
% 4 at a time and create a "snapshot"

newScoresFig = plotgui('new', myScores,...
  'columns', 'viewclasses', 1);plotgui('menuselection', 'ViewSubplots4')
% the last plotgui call above is what creates a 2 x 2 arrangement of scores
% plots
legend('show', 'Location', 'best');
newScoresFig.Position = [500 100 1220 780];

myAxes = findobj(newScoresFig, 'type', 'axes');

% the axis objects are ordered in reverse order left => right, top =>
% bottom

% so myAxes(4) is in the upper left hand corner, myAxes(3) is in the upper
% right hand corner, . . .

% set up the video collection

vidObj  = VideoWriter('myMovieII.mp4', 'MPEG-4');
vidObj.FrameRate = 0.25;  % modify as desired
vidObj.Quality = 90;  %modify as desired
open(vidObj);
colInds = 1:4;

for frameCounter = 1:11
  curColInds = 4*(frameCounter-1) + colInds;
  for axisCounter = 1:4
    legend('off');
    setappdata(myAxes(5-axisCounter), ...
      'axismenuvalues', {0 curColInds(axisCounter) 0});
  end
  plotgui('update', 'figure', newScoresFig);
  legend('show', 'Location', 'best');
  drawnow
  pause(2)
  curFrame = getframe(newScoresFig);
  writeVideo(vidObj, curFrame);
end
    
close(vidObj);    



%% Profiling and Optimizing

% Use tic/toc to get a meaure of how long your code takes:
tic;
for i = 1:1000000
    my_array(i) = rand(1) * i;
end
toc;

% Try again with preallocation.
tic;
length = 1000000;
my_array = zeros(1, length);

for i = 1:length
    my_array(i) = rand(1) * i;
end
toc;

% Let's look at a profile.
close all
clear

profile on

mydata = magic(100);
plot(magic(100))

% Stops profile and opens viewer. 
profile viewer

% Look at Flame Graph, organized from bottom up.
% Click on function (newplot) in table. 
%   Look at code, see that gcf and gca calls are the


% Try again but pull out the figure and axis creation.
close all
ax = axes;

profile on

mydata = magic(100);
plot(ax,mydata)

profile viewer

%% Model Optimizer

% The modeloptimizerdemo script does a good job of showing how to create
% snapshots and assemble the optimizer. Try running it and see how it
% works:

modeloptimizerdemo

%% Other resources for scripting 

% Matlab performance techniques:
web('https://www.mathworks.com/help/matlab/matlab_prog/techniques-for-improving-performance.html')

% Our Intro to Matlab course script:
edit introtomatlab.m

% Our Managing Object course script:


%% MCCT example

% in Live Script file

